tic

fprintf('\nEstimating %s...\n',modelStr)

% Need to set whichAlgo option to use initialComp.m. However, note that
% both Algorithm 1 and the approach from GMM18 are used to compute the
% bounds of the identified set below.
opt.whichAlgo = 0;
% Compute useful variables.
[opt.calcLRCIR,oneColQ,restr.f,restr.s] = initialComp(N,opt,restr);

% Storage matrices.
rMinPostGK = zeros(opt.Kdraws,opt.H+1,length(opt.ivar));
rMaxPostGK = rMinPostGK;
rMinPostGMM = rMinPostGK;
rMaxPostGMM = rMinPostGK;
rSinglePriorPost = rMinPostGK;
B = zeros(N*M,opt.Kdraws);
Sigma = zeros(N,N,opt.Kdraws);

phiDraw = 0; % Counter for no. of draws from posterior of phi
nonStableCount = 0; % Counter for no. of draws with nonstable VAR
nonEmptyDraw = 0; % Counter for no. of draws with non-empty identified set

while nonEmptyDraw < opt.Kdraws

    % Posterior sampler with noninformative improper (Jeffreys) prior.
    phi.Sigma = iwishrnd(phiHat.S,T-M);
    phi.Sigmatr = chol(phi.Sigma,'lower');
    phi.Sigmatrinv = phi.Sigmatr\eye(N);
    phi.B = phiHat.B(:) + kron(phi.Sigmatr,phiHat.cholP)*randn(M*N,1);
    
    % Generate coefficients in reduced-form VMA representation and, if 
    % required, long-run (cumulative) IR. Also generate versions of these 
    % objects post-multiplied by Sigmatr, and check invertibility of the 
    % VAR representation.
    [irs,nonStable] = genVMA(phi,opt);
    nonStableCount = nonStableCount + nonStable;

    % If VAR is stable, proceed. Otherwise, redraw phi.
    
    if nonStable == 0
        
    phiDraw = phiDraw + 1;    
    
    % Use analytical approach to determine whether identified set is
    % nonempty.
    QemptyExact(phiDraw) = checkEmptyISAnalytical(restr,irs,phi,opt);
    
    % Use Algorithm 1 (Step 2) to determine whether identified set is
    % nonempty and draw Q0 from space of orthonormal matrices
    % satisfying equality and sign restrictions. This draw is used to
    % compute the IR under a single prior for Q.
    [Q0,restr.F,restr.S,Qempty(phiDraw)] = drawQ(restr,irs,phi,opt);
    
    if Qempty(phiDraw) == 0 % If identified set non-empty

            % Store reduced-form parameters
            nonEmptyDraw = nonEmptyDraw + 1;
            Sigma(:,:,nonEmptyDraw) = phi.Sigma;
            B(:,nonEmptyDraw) = phi.B;

            % Compute draw from posterior of IRs given single prior for Q.

            for hh = 1:opt.H+1 % For each horizon

                for ii = 1:Ni % For each variable of interest

                    rSinglePriorPost(nonEmptyDraw,hh,ii) = ...
                        irs.vmaGK(opt.ivar(ii),:,hh)*Q0(:,opt.jshock);

                end

            end
            
           % Find upper and lower bounds of IS of the IRs.
           
           % Use Algorithm 1 (constrained optimisation).
           % Users without the Parallel Computing Toolbox can replace 
           % the function numericalBoundsPar with numericalBounds.
           [rMinPostGK(nonEmptyDraw,:,:),rMaxPostGK(nonEmptyDraw,:,:)] = ...
               numericalBoundsPar(restr,irs,phi,opt,optimOptions);

           % Use analytical results from GMM18.
           [rMinPostGMM(nonEmptyDraw,:,:),rMaxPostGMM(nonEmptyDraw,:,:)] = ...
               analyticalBounds(restr,irs,phi,opt);
              
           if mod(opt.Kdraws-nonEmptyDraw,opt.dispIter) == 0
              
               fprintf('\n%d draws with non-empty identified set remaining...',...
               opt.Kdraws-nonEmptyDraw);
               
           end      
        
    end    
           
    end
    
end

% Compute posterior mean under single prior.
rSinglePriorMean = permute(mean(rSinglePriorPost,1),[2 3 1]);

% Compute posterior mean bounds.
meanlbGK = permute(mean(rMinPostGK,1),[2 3 1]);
meanubGK = permute(mean(rMaxPostGK,1),[2 3 1]);
meanlbGMM = permute(mean(rMinPostGMM,1),[2 3 1]);
meanubGMM = permute(mean(rMaxPostGMM,1),[2 3 1]);

% Compute robustified credible region for identified set.
[credlbGK,credubGK] = credibleRegion(rMinPostGK,rMaxPostGK,opt);
[credlbGMM,credubGMM] = credibleRegion(rMinPostGMM,rMaxPostGMM,opt);
% Compute highest posterior density interval under single prior.
[hpdlb,hpdub] = highestPosteriorDensity(rSinglePriorPost,opt);

% Compute posterior plausibility of the identifying restrictions.
postPlaus = 1 - sum(Qempty)/length(Qempty);
fprintf('\n\nPosterior plausibility of identifying restrictions (approximate): %0.4g \n',...
    postPlaus);
postPlausExact = 1 - sum(QemptyExact)/length(QemptyExact);
fprintf('\nPosterior plausibility of identifying restrictions (exact): %0.4g \n',...
    postPlausExact);

% Width of posterior mean bounds.
postMeanBoundWidthGK = meanubGK - meanlbGK;
postMeanBoundWidthGMM = meanubGMM - meanlbGMM;
hpdWidth = hpdub - hpdlb; % Width of highest posterior density regions
% Width of robustified credible region.
credWidthGK = credubGK - credlbGK; 
credWidthGMM = credubGMM - credlbGMM; 

% Informativeness of prior.
priorInformativenessGK = 1 - hpdWidth./credWidthGK; 
priorInformativenessGMM = 1 - hpdWidth./credWidthGMM;

% Compute informativeness of identifying restrictions if computeIR == 1.
% This requires the width of the identified set in the model with no
% restrictions (Model 0) to have been computed previously.
if computeIR == 1 
    
   restrictionInformativenessGK = ...
       1 -  postMeanBoundWidthGK./postMeanBoundWidthGKModel0;
   restrictionInformativenessGMM = ...
       1 -  postMeanBoundWidthGMM./postMeanBoundWidthGMMModel0;   
    
end

% Proportion of draws where VAR is noninvertible.
nonInvertPc = nonStableCount/phiDraw*100;
fprintf('\nVAR representation noninvertible in %0.3f per cent of draws. \n',...
    nonInvertPc);

diffMin = abs(rMinPostGK - rMinPostGMM);
diffMax = abs(rMaxPostGK - rMaxPostGMM);

runTime = toc

% Save results.
fileName = strcat(modelStr,'.mat');
save(fileName);

clear Qempty QemptyExact;