%% Replicate Tables from GK.
% Output is appropriate for copying into LaTeX.
% Run replicateGK.m first.

horizons = [1 10 20]; % Horizons of interest
headers = {sprintf('h%d',horizons(1)),sprintf('h%d',horizons(2)),...
    sprintf('h%d',horizons(3))};

str = 'Model';

for mod = 1:8
   
   modelStr = sprintf('%s%d',str,mod-1);
   
   load(modelStr); % Load results
   
   TT = cell(7,7);
   TT(:,:) = {'&'};
   TT(:,1) = {'SB: posterior mean',...
       'SB: 90\% credible region',...
       'RB: set of posterior means',...
       'RB: 90\% robust credible region',...
       'Lower probability: $\pi_{\eta|Y*}(\eta<0)^{\dagger}$'...
       'Informativeness of restrictions*',...
       'Informativeness of prior**'};
   
   for jj = 1:length(horizons)
       
       hh = horizons(jj);
   
       TT{1,(jj-1)*2+3} = num2str(rSinglePriorMean(hh+1),'%2.2f');
       a = strcat('[',cellstr(num2str(hpdlb(hh+1),'%2.2f')),...
           ',',cellstr(num2str(hpdub(hh+1),'%2.2f')),']');
       TT{2,(jj-1)*2+3} = a{1};
       a = strcat('[',cellstr(num2str(meanlbGK(hh+1),'%2.2f')),...
           ',',cellstr(num2str(meanubGK(hh+1),'%2.2f')),']');
       TT{3,(jj-1)*2+3} = a{1};
       a = strcat('[',cellstr(num2str(credlbGK(hh+1),'%2.2f')),...
            ',',cellstr(num2str(credubGK(hh+1),'%2.2f')),']');
       TT{4,(jj-1)*2+3} = a{1};
       postProbIrfNegLB = sum(rMaxPostGK < 0,1)/opt.Kdraws;
       TT{5,(jj-1)*2+3} = num2str(postProbIrfNegLB(hh+1),'%1.3f');
       
       if mod == 1 % Model 0
           
           TT{6,(jj-1)*2+3} = {'\textendash'};
           
       else
       
           TT{6,(jj-1)*2+3} = num2str(restrictionInformativenessGK(hh+1),...
                                        '%1.2f');
       end
       
       TT{7,(jj-1)*2+3} = num2str(priorInformativenessGK(hh+1),'%1.2f'); 
        
   end
   
   fileName = strcat(modelStr,'_Table.mat');
   save(fileName,'TT');
     
end

lastCol = cell(7,1);
lastCol(1:end-1) = {'\\'};
lastCol(end) = {'\\ \hline\hline'};

load('Model0_Table');
T0 = TT;
load('Model1_Table');
T1 = TT;
TComb01 = [T0 T1(:,2:end) lastCol];

load('Model2_Table');
T2 = TT;
load('Model3_Table');
T3 = TT;
TComb23 = [T2 T3(:,2:end) lastCol];

load('Model4_Table');
T4 = TT;
load('Model5_Table');
T5 = TT;
TComb45 = [T4 T5(:,2:end) lastCol];

load('Model6_Table');
T6 = TT;
load('Model7_Table');
T7 = TT;
TComb67 = [T6 T7(:,2:end) lastCol];

space = cell(1,14);

TComb = [TComb01; space; TComb23; space; TComb45; space; TComb67];

xlswrite('Table2.xlsx',TComb);

%% Extract information on posterior plausibility.
posteriorPlausibility = cell(2,16);
posteriorPlausibility(:) = {'&'};

for modi = 1:8
    
   modelStr = sprintf('%s%d',str,modi-1);
   
   load(modelStr); % Load results
   
   posteriorPlausibility(1,(modi-1)*2+2) = ...
       cellstr(num2str(postPlaus,'%1.4f')); 
   posteriorPlausibility(2,(modi-1)*2+2) = ...
       cellstr(num2str(postPlausExact,'%1.4f'));    
   
    
end

% Estimates of posterior plausibility are the same across the two
% approaches, so just output one of them.

posteriorPlausibility = ...
    ['$\tilde{\pi}_{\phi|Y(\{\phi:IS_{\eta}(\phi)\neq \emptyset\})$' ...
    posteriorPlausibility(2,:) '\\ \hline\hline'];

xlswrite('Table1_posteriorPlausibility.xlsx',posteriorPlausibility);

delete Model*_Table.mat